@@ -1,5 +1,6 @@
 import asyncio
 import logging
+import socket
 import websockets
 from gabriel_protocol import gabriel_pb2
 from collections import namedtuple
@@ -18,6 +19,17 @@ websockets_logger.setLevel(logging.INFO)
 ProducerWrapper = namedtuple('ProducerWrapper', ['producer', 'source_name'])
 
 
+# It isn't necessary to do this as of Python 3.6 because
+# "The socket option TCP_NODELAY is set by default for all TCP connections"
+# per https://docs.python.org/3/library/asyncio-eventloop.html
+# However, this seems worth keeping in case the default behavior changes.
+class NoDelayProtocol(websockets.client.WebSocketClientProtocol):
+    def connection_made(self, transport: asyncio.BaseTransport) -> None:
+        super().connection_made(transport)
+        sock = transport.get_extra_info('socket')
+        sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+
+
 class WebsocketClient:
     def __init__(self, host, port, producer_wrappers, consumer):
         '''
@@ -34,16 +46,22 @@ class WebsocketClient:
         self.producer_wrappers = producer_wrappers
         self.consumer = consumer
 
-    def launch(self):
+    def launch(self, message_max_size=None):
         event_loop = asyncio.get_event_loop()
 
         try:
             self._websocket = event_loop.run_until_complete(
-                websockets.connect(self._uri))
+                websockets.connect(self._uri, create_protocol=NoDelayProtocol,
+                                   max_size=message_max_size))
         except ConnectionRefusedError:
             logger.error('Could not connect to server')
             return
 
+        # We don't waste time checking TCP_NODELAY in production.
+        # Note that websocket.transport is an undocumented property.
+        # sock = self._websocket.transport.get_extra_info('socket')
+        # assert(sock.getsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY) == 1)
+
         consumer_task = asyncio.ensure_future(self._consumer_handler())
         tasks = [
             asyncio.ensure_future(self._producer_handler(
@@ -155,7 +173,7 @@ class _Source:
     async def get_token(self):
         while self._num_tokens < 1:
             logger.debug('Waiting for token')
-            self._event.clear()  # Clear because we definitely want wait
+            self._event.clear()  # Clear because we definitely want to wait
             await self._event.wait()
 
         self._num_tokens -= 1
